%% A short YALMIP demo for grid-connected PV-battery system self-consumption
% Pierre Haessig - April 2017
% version with blanks (Objective and Constraints definition)

%% Problem definition
% Time discretization of one day
T = 24; % hours
N = T*2;
dt = T/N;
t = (0:(N-1))'*dt; % in hours

% System Sizing:
Ppv_max = 2; % kW
Pgrid_max = 3; % kW
Psto_max = 2; % kW
Esto_max = 4; % kWh

% PV profile: between 6:00 and 18:00. Max at 12:00.
% more data from http://re.jrc.ec.europa.eu/pvgis/apps4/pvest.php#
Ppv = (1-0.03*(mod(t, 24)-12).^2);
Ppv(Ppv<0) = 0;
Ppv = Ppv*Ppv_max;

% Load profile
%Pload = 0.8 + 0*t; % constant
Pload = Pload_enedis()*0.7;

% Optim variables
Pgrid = sdpvar(N,1);
Psto = sdpvar(N,1);
Esto = sdpvar(N+1,1);
Pshed = sdpvar(N,1);

% Optim problem: grid energy + small penalty on storage usage
Objective = sum(Pgrid)*dt + 0.01*sum(Psto.^2)*dt;

Constraints = [
    Pgrid >=  0, Pgrid <= Pgrid_max,
    Psto >= -Psto_max,  Psto <= Psto_max,
    Esto >=  0, Esto <= Esto_max,
    Pshed >= 0,
    
    Esto(2:end) == Esto(1:end-1) + Psto*dt % Storage energy balance
    Esto(1) == Esto(end) % Storage Periodicity
    Ppv + Pgrid == Pload+Psto+Pshed % Power flow balance
    ];

%% Solve
sol = optimize(Constraints,Objective);
disp(sol)
value(Objective)

%% Extract values
Pgrid_v = value(Pgrid);
Esto_v = value(Esto);
Psto_v = value(Psto);
Pshed_v = value(Pshed);

fprintf('Energy exchanges on the day:\n')
fprintf('E load: %5.2f kWh\n', sum(Pload)*dt)
fprintf('E pv:   %5.2f kWh = %.1f%% load (before shedding)\n', sum(Ppv)*dt, sum(Ppv)/sum(Pload)*100)
fprintf('E pv:   %5.2f kWh = %.1f%% load (after shedding)\n', sum(Ppv-Pshed_v)*dt, sum(Ppv-Pshed_v)/sum(Pload)*100)
fprintf('E grid: %5.2f kWh = %.1f%% load \n', sum(Pgrid_v)*dt, sum(Pgrid_v)/sum(Pload)*100)

% Output snapshot: Energy exchanges on the day:
% E load: 15.72 kWh
% E pv:   15.41 kWh = 98.0% load (before shedding)
% E pv:   11.29 kWh = 71.8% load (after shedding)
% E grid:  4.43 kWh = 28.2% load 

%% Plot trajectory
subplot(311)
hold on

plot(t, Ppv)
plot(t, Pload)
legend('pv', 'load')
ylabel('Power (kW)')
title('Grid connected PV for self-consumption')
grid on;

subplot(312)
hold on
plot(t, Pgrid_v)
plot(t, Pshed_v)
plot(t, Psto_v)
ylabel('Power (kW)')
legend('grid', 'shed', 'sto')
grid on;

subplot(313)
plot(t, Esto_v(1:end-1))
ylim([-0.05*Esto_max, 1.05*Esto_max])
grid on
legend('Esto')
xlabel('t (hours)')
ylabel('Energy (kWh)')
